attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute mediump vec3 a_tangent;

uniform highp mat4 u_ModelViewProj;
uniform highp mat4 u_ModelView;
uniform highp vec3 u_eyePosition;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform lowp vec3 u_lightSpecular;
uniform highp vec3 u_lightPosition;
uniform mediump float u_lightConstantAttenuation;

uniform lowp vec3 u_materialAmbient;
uniform lowp vec3 u_materialDiffuse;
uniform lowp vec3 u_materialSpecular;
uniform lowp vec3 u_materialEmission;
uniform mediump float u_materialShininess;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying lowp vec3 v_Ambient;
varying lowp vec3 v_Diffuse;
varying lowp vec3 v_Specular;

varying lowp vec3 v_Light;
varying lowp vec3 v_Eye;

uniform highp float u_fogParam1;
uniform highp float u_fogParam2;
uniform lowp vec4 u_fogColor;
varying lowp float v_fogFactor;
varying lowp vec4 v_fogColor;


void main()
{
 	// 頂点を視点空間に座標変換する
	highp float z = (u_ModelView * a_vertex).z;

	lowp vec3 E = normalize( u_eyePosition - a_vertex.xyz );
	lowp vec3 L = normalize( -u_lightPosition );

	lowp vec3 N = normalize( a_normal );
	lowp vec3 T = a_tangent;
	lowp vec3 B = cross( N, T );
	
	v_color = a_color + vec4( u_materialEmission, 1.0 );
	v_texCoord = a_texCoord;

	lowp float attenuation = 1.0 / u_lightConstantAttenuation;
	attenuation = 1.0;
	
	v_Ambient = u_materialAmbient * u_lightAmbient;
	v_Diffuse = u_materialDiffuse * u_lightDiffuse * attenuation;
	v_Specular = u_materialSpecular * u_lightSpecular * attenuation;

	v_Eye.x = dot(E, T);
	v_Eye.y = dot(E, B);
	v_Eye.z = dot(E, N);

	v_Light.x = dot(L, T);
	v_Light.y = dot(L, B);
	v_Light.z = dot(L, N);
	
	// フォグ係数を[0,1]で作成
	v_fogFactor = clamp( u_fogParam1 + u_fogParam2 * z, 0.0, 1.0 );
	
	// フォグ係数とフォグの色から視点からの距離による色を作成
	v_fogColor = u_fogColor * ( 1.0 - v_fogFactor );
	
	gl_Position = u_ModelViewProj * a_vertex;
}
